package org.bitbucket.bradleysmithllc.etlunit.io.file;

import org.bitbucket.bradleysmithllc.etlunit.DiffManager;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestOperation;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Map;

public interface DataFileManager
{
	String DEFAULT_COLUMN_DELIMITER = "\t";
	String DEFAULT_ROW_DELIMITER = "\n";
	String DEFAULT_NULL_TOKEN = "null";

	DataFileSchema.format_type DEFAULT_FORMAT_TYPE = DataFileSchema.format_type.delimited;

	void setDefaultColumnDelimiter(String delimiter);

	void setDefaultRowDelimiter(String delimiter);

	void setDefaultNullToken(String token);

	void setDefaultFormatType(DataFileSchema.format_type type);

	String getDefaultColumnDelimiter();

	String getDefaultRowDelimiter();

	String getDefaultNullToken();

	DataFileSchema.format_type getDefaultFormatType();

	/**
	 * Loads the schema from the specified resource path.  The default class loader is to use
	 * the context class loader of the current thread.
	 * @param resourcePath
	 * @return
	 */
	DataFileSchema loadDataFileSchemaFromResource(String resourcePath);

	/**
	 * Loads the schema from the specified resource path.  The specified class loader is used.  Use this
	 * method sparingly since this build runs in a java application as well as a maven plugin.
	 * @param resourcePath
	 * @param classLoaderContext
	 * @return
	 */
	DataFileSchema loadDataFileSchemaFromResource(String resourcePath, Object classLoaderContext);

	DataFileSchema createDataFileSchema(String id);

	DataFileSchema loadDataFileSchema(File schema);

	DataFile loadDataFile(File file, DataFileSchema schema);

	/**
	 * Compares l-file to r-file, using the column list in lfile as the basis for comparison.  Any
	 * additional columns in the r-file will be ignored.   Both files must be ordered according to the
	 * order key (defined by the order columns in the schema).
	 *
	 * @param lfile - In practice this is the 'actual'
	 * @param rfile - This should be the 'expected'
	 * @return
	 */
	List<FileDiff> diff(DataFile lfile, DataFile rfile) throws IOException;

	/**
	 * Compares l-file to r-file, using the column list in lfile as the basis for comparison.  Any
	 * additional columns in the r-file will be ignored.   Both files must be ordered according to the
	 * order key (defined by the order columns in the schema).
	 *
	 * @param lfile - In practice this is the 'actual'
	 * @param rfile - This should be the 'expected'
	 * @param columns - A list of columns to constrain the comparison to
	 * @return
	 */
	List<FileDiff> diff(DataFile lfile, DataFile rfile, List<String> columns) throws IOException;

	/**
	 *
	 * @param dManager
	 * @param operation
	 * @param failureId
	 * @param diffs
	 * @param expectedId Identity of the expected data set
	 * @param actualId Identity of the actual data set
	 */
	void report(DiffManager dManager, ETLTestOperation operation, String failureId, List<FileDiff> diffs, String expectedId, String actualId);

	DataFormatValidator resolveValidatorById(String id);

	/**
	 * Copies a data file from one schema to another.  These schemas must be compatible
	 * or else the copy will fail (E.G., if there is no intersection between the two schemas
	 * then the resulting file will be all nulls.
	 *
	 * @param source
	 * @param dest
	 */
	void copyDataFile(DataFile source, DataFile dest, Map<String, String> defaultValues) throws IOException;

	void copyDataFile(DataFile source, DataFile dest) throws IOException;
}